@@ -7,9 +7,10 @@ from pkg_resources import resource_filename
 from psutil import process_iter
 from uno import createUnoStruct
 from unogenerator.reusing.listdict_functions import listdict_min
+from time import sleep
 
-__version__ = '0.12.0'
-__versiondatetime__=datetime(2021, 11, 14, 19, 49)
+__version__ = '0.13.0'
+__versiondatetime__=datetime(2021, 11, 17, 7, 54)
 __versiondate__=__versiondatetime__.date()
 
 try:
@@ -439,30 +440,40 @@ def get_range_from_iterable_object( coord_start, o):
         len_columns=len(o)-1
     return Range(f"{coord_start.string()}:{coord_start.addRowCopy(len_rows).addColumnCopy(len_columns).string()}")
 
-## Returns a dictt with process info
-def get_from_process_info(cpu_percentage=False):    
-    try:
-        r=[]
-        for p in process_iter(['name','cmdline', 'pid']): 
-            d={}
-            if p.info['name']=='soffice.bin':
-                if  'file:///tmp/unogenerator'  in ' '.join(p.info['cmdline']):
-                    d["port"]=int(p.info['cmdline'][1][-4:])
-                    d["pid"]=p.pid
-                    d["mem"]=p.memory_info().rss
-                    d["cpu_number"]=p.cpu_num()
-                    if cpu_percentage is True:
-                        d["cpu_percentage"]=p.cpu_percent(interval=0.01)
-                        d["object"]=p
-                    r.append(d)
-        return r
-    except:
-        print(_("Have you launched unogenerator instances?. Please run unogenerator_start"))
-        return []
+## @param attempts (Integer). Sometimes when server is busy this method fails to detect info. So I make several attempts with a time interval
+## @return listdict with process info
+
+def get_from_process_info(cpu_percentage=False, attempts=10):
+    for attempt in range(attempts):
+        try:
+            r=[]
+            for p in process_iter(['name','cmdline', 'pid']): 
+                d={}
+                if p.info['name']=='soffice.bin':
+                    if  'file:///tmp/unogenerator'  in ' '.join(p.info['cmdline']):
+                        d["port"]=int(p.info['cmdline'][1][-4:])
+                        d["pid"]=p.pid
+                        d["mem"]=p.memory_info().rss
+                        d["cpu_number"]=p.cpu_num()
+                        if cpu_percentage is True:
+                            d["cpu_percentage"]=p.cpu_percent(interval=0.01)
+                            d["object"]=p
+                        r.append(d)
+            if len(r)==0:
+                raise
+            return r
+        except:
+            print(_(f"I couldn't detect unogenerator process info ({attempt}/{attempts} attempts)"))
+            sleep(5)
+            continue
+    print(_("Have you launched unogenerator instances?. Please run unogenerator_start"))
+    return []
     
 
 def get_from_process_numinstances_and_firstport():        
     ld=get_from_process_info()
+    if len(ld)==0:
+        print(_("I couldn't detect unogenerator instances"))
     return len(ld), listdict_min(ld,"port")
    
 def red(s):
