@@ -3,10 +3,14 @@
 
 from datetime import datetime
 from os import path, makedirs
-from uno import getComponentContext, createUnoStruct, systemPathToFileUrl
+from uno import getComponentContext, createUnoStruct, systemPathToFileUrl, Any
 from com.sun.star.beans import PropertyValue
 from com.sun.star.text import ControlCharacter
 from com.sun.star.awt import Size
+from com.sun.star.sheet.ConditionEntryType import COLORSCALE
+
+
+
 from com.sun.star.style.ParagraphAdjust import RIGHT,  LEFT
 from com.sun.star.style.BreakType import PAGE_BEFORE, PAGE_AFTER
 from gettext import translation
@@ -88,6 +92,16 @@ class ODF:
             styles.sort()
             for style in styles:
                 print ( f"    - {style}")
+                
+                
+    # Returns a list of page styles objects
+    def getPageStyles(self):
+        r=[]
+        family=self.document.StyleFamilies.getByName('PageStyles')
+        for i in range(family.Count):
+            r.append(family.getByIndex(i))
+        return r
+        
 
     def setLanguage(self, language, country):
         print("FALTA")
@@ -183,25 +197,44 @@ class ODT(ODF):
             makedirs(path.dirname(path.abspath(filename)), exist_ok=True)
             copyfile(tempfile, filename)
 
-##def insertTextIntoCell( table, cellName, text, color ):
-##    tableText = table.getCellByName( cellName )
-##    cursor = tableText.createTextCursor()
-##    cursor.setPropertyValue( "CharColor", color )
-##    tableText.setString( text )
-    def find_and_replace(self, find, replace=""):
+    ## This method has problems with ().\ *// especial characters. Findall and replace works fine
+    ## Last is the descriptor returned after first found
+    ## @return None if  not found. Descriptor(can be used again with last) of the last found
+    def find_and_replace(self, find, replace="", last=None, log=False):
         search=self.document.createSearchDescriptor()
         search.SearchString=find
-        found=self.document.findFirst(search)
+        if last is None:
+            found=self.document.findFirst(search)
+        else:
+            found=self.document.findNext(last,search)
+        
         if found is not None:
             found.setString("")
             self.cursor=found
             found.Text.insertString(self.cursor, replace, False)
         else:
-            warning(f"'{find}' was not found in the document'")
-            
+            if log is True:
+                warning(f"'{find}' was not found in the document'")
+        return found
+
+    def findall_and_replace(self, find, replace="", log=False):
+        search=self.document.createReplaceDescriptor()
+        search.setSearchString(find)
+        search.setReplaceString(replace)
+        found=self.document.findFirst(search)
+        if found is not None:
+            number=self.document.replaceAll(search)
+            debug(f"Replaced {number} times '{find}' by '{replace}'")
+            self.cursor=found
+            return number
+        else:
+            if log is True:
+                debug(f"'{find}' was not found in the document'")
+            return 0
+
     ## Sets the cursor after found string
-    def find(self, find):
-        self.first_and_replace(find, find)
+    def find(self, find, log=False):
+        self.first_and_replace(find, find, log)
 
     ## Find a string and deleteAll untill the end of the document
     ## Usefull to delete document part with styles in templates
@@ -217,14 +250,6 @@ class ODT(ODF):
             oVC.gotoEnd(False)
             self.cursor.gotoRange(oVC,True)			#'Move Text Cursor to same location as oVC while selecting text in between (True)
             self.cursor.setString("")
-#            self.document.Text.insertString(self.cursor, "FIN2", False)
-            
-##            while True:
-#            self.cursor.gotoEnd(True)
-##                r=self.cursor.gotoNextParagraph()
-#                if r==False:
-#                    break
-#            found.Text.insertString(self.cursor, "AQUI", False)
         else:
             warning(f"'{find}' was not found in the document'")
         
@@ -379,6 +404,8 @@ class ODT(ODF):
         self.cursor.setPropertyValue("PageDescName", style)
         self.document.Text.insertString(self.cursor, "", False)
 
+
+
 class ODS(ODF):
     def __init__(self, template=None, loserver_port=2002):
         ODF.__init__(self, template, loserver_port)
@@ -490,7 +517,7 @@ class ODS(ODF):
             self.addColumnWithStyle(coord_start.addColumnCopy(i), column, colors=colors,styles=styles)
 
     ## @param style If None tries to guess it
-    @timeit
+#    @timeit
     def addCellWithStyle(self, coord, o, color=ColorsNamed.White, style=None):
         coord=C.assertCoord(coord)
         if style is None:
@@ -500,6 +527,8 @@ class ODS(ODF):
         cell.setPropertyValue("CellStyle", style)
         cell.setPropertyValue("CellBackColor", color)
         self.numcells=self.numcells+1
+        if self.numcells % 500==0:
+            debug(f"Wrote {self.numcells} cells in {datetime.now()-self.init}")
         
     ## All of this names are document names
     def setCellName(self, coord, name):
@@ -534,6 +563,23 @@ class ODS(ODF):
         else:
             cell.setString(str(o))
             print("MISSING", o.__class__.__name__)
+            
+    def sortRange(self, range,  sortindex, ascending=True, casesensitive=True):
+        range=R.assertRange(range)
+        unorange=self.sheet.getCellRangeByName(range.string())
+        sortfield=createUnoStruct("com.sun.star.table.TableSortField")
+        sortfield.Field=sortindex
+        sortfield.IsAscending=ascending
+        sortfield.IsCaseSensitive=casesensitive
+        sortDescr = unorange.createSortDescriptor()
+        
+
+        for prop in sortDescr:
+            if prop.Name == 'SortFields':
+                prop.Value = Any('[]com.sun.star.table.TableSortField', (sortfield,))
+        # sort ...
+        unorange.sort(sortDescr)
+        
 
     def showStatistics(self):
         debug(f"- Number of cells {self.numcells}")
@@ -745,7 +791,26 @@ class ODS(ODF):
             self.document.storeToURL(systemPathToFileUrl(tempfile), args)
             makedirs(path.dirname(path.abspath(filename)), exist_ok=True)
             copyfile(tempfile, filename)
-
+            
+    def setColorScale(self, range):
+        range=R.assertRange(range)
+        myCells=self.sheet.getCellRangeByName(range.string())
+#        print(unorange, dir(unorange))
+#        print(self.sheet, dir(self.sheet))
+        myConditionalFormat = myCells.ConditionalFormat
+        args=(
+            PropertyValue('Operator',0, COLORSCALE,0),
+        )
+        myConditionalFormat.clear()
+        myConditionalFormat.addNew(args)
+        myCells.ConditionalFormat = myConditionalFormat
+#        self.ws_current.conditional_formatting.add(range, 
+#                            openpyxl.formatting.rule.ColorScaleRule(
+#                                                start_type='percentile', start_value=0, start_color='00FF00',
+#                                                mid_type='percentile', mid_value=50, mid_color='FFFFFF',
+#                                                end_type='percentile', end_value=100, end_color='FF0000'
+#                                                )
+#                                            )
 
 class ODS_Standard(ODS):
     def __init__(self, loserver_port=2002):
